# [[[ HEADER ]]]
use RPerl;
package RPerl::Test::LiteralNumber::Package_00_Good;
use strict;
use warnings;
our $VERSION = 0.001_000;

# [[[ SUBROUTINES ]]]
our integer $empty_sub = sub {
    return 2;
};

1;    # end of package
