# [[[ HEADER ]]]
package RPerl::DataType::Modifier;
use strict;
use warnings;
use RPerl::AfterSubclass;
our $VERSION = 0.001_010;

# NEED FIX?: RPerl Data Type, no inheritance from RPerl or RPerl::CompileUnit::Module::Class

# [[[ OO INHERITANCE ]]]
#use parent qw(RPerl::CompileUnit::Module::Class);
#use RPerl::CompileUnit::Module::Class;

1;  # end of class
