# [[[ HEADER ]]]
package RPerl::DataStructure::CodeReference;
use strict;
use warnings;
use RPerl::AfterSubclass;
our $VERSION = 0.001_000;

# [[[ OO INHERITANCE ]]]
use parent qw(RPerl::DataStructure);
use RPerl::DataStructure;

# [[[ OO PROPERTIES ]]]
our hashref $properties = {};

1;    # end of class